@@ -79,9 +79,9 @@ bool CVLCommandHandler::InitHandler(void)
       {
         libcec_configuration config;
         m_processor->GetPrimaryClient()->GetCurrentConfiguration(config);
-        if (config.iDoubleTapTimeout50Ms == 0)
+        if (config.iDoubleTapTimeoutMs == 0)
         {
-          config.iDoubleTapTimeout50Ms = config.clientVersion >= LIBCEC_VERSION_TO_UINT(2, 2, 0) ? CEC_DOUBLE_TAP_TIMEOUT_50_MS : CEC_DOUBLE_TAP_TIMEOUT_MS_OLD;
+          config.iDoubleTapTimeoutMs = CEC_DOUBLE_TAP_TIMEOUT_MS;
           m_processor->GetPrimaryClient()->SetConfiguration(config);
         }
 
